@@ -1,4 +1,4 @@
-#    (c) Copyright 2016 Brocade Communications Systems Inc.
+#    (c) Copyright 2019 Brocade, a Broadcom Company
 #    All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -26,12 +26,12 @@ from eventlet import greenthread
 from oslo_concurrency import processutils
 from oslo_log import log as logging
 from oslo_utils import excutils
-import six
 
 from cinder import exception
-from cinder.i18n import _, _LE
+from cinder.i18n import _
 from cinder import ssh_utils
 from cinder import utils
+from cinder.zonemanager.drivers.brocade import exception as b_exception
 import cinder.zonemanager.drivers.brocade.fc_zone_constants as zone_constant
 
 LOG = logging.getLogger(__name__)
@@ -43,7 +43,7 @@ class BrcdFCZoneClientCLI(object):
     switch_user = 'admin'
     switch_pwd = 'none'
     switch_key = 'none'
-    patrn = re.compile('[;\s]+')
+    patrn = re.compile(r'[;\s]+')
 
     def __init__(self, ipaddress, username,
                  password, port, key):
@@ -82,10 +82,10 @@ class BrcdFCZoneClientCLI(object):
         try:
             switch_data = self._get_switch_info(
                 [zone_constant.GET_ACTIVE_ZONE_CFG])
-        except exception.BrocadeZoningCliException:
+        except b_exception.BrocadeZoningCliException:
             with excutils.save_and_reraise_exception():
-                LOG.error(_LE("Failed getting active zone set "
-                              "from fabric %s"), self.switch_ip)
+                LOG.error("Failed getting active zone set "
+                          "from fabric %s", self.switch_ip)
         try:
             for line in switch_data:
                 line_split = re.split('\\t', line)
@@ -162,7 +162,7 @@ class BrcdFCZoneClientCLI(object):
                 'zone_members_with_sep': zone_members_with_sep}
             LOG.debug("Creating zone, cmd to run %s", cmd)
             self.apply_zone_change(cmd.split())
-            if(iterator_count > 0):
+            if (iterator_count > 0):
                 zone_with_sep += ';'
             iterator_count += 1
             zone_with_sep += zone
@@ -194,9 +194,9 @@ class BrcdFCZoneClientCLI(object):
             self._cfg_trans_abort()
             msg = _("Creating and activating zone set failed: "
                     "(Zone set=%(cfg_name)s error=%(err)s)."
-                    ) % {'cfg_name': cfg_name, 'err': six.text_type(e)}
+                    ) % {'cfg_name': cfg_name, 'err': str(e)}
             LOG.error(msg)
-            raise exception.BrocadeZoningCliException(reason=msg)
+            raise b_exception.BrocadeZoningCliException(reason=msg)
 
     def update_zones(self, zones, activate, operation, active_zone_set=None):
         """Update the zone configuration.
@@ -243,7 +243,7 @@ class BrcdFCZoneClientCLI(object):
                 'zone_members_with_sep': zone_members_with_sep}
             LOG.debug("Updating zone, cmd to run %s", cmd)
             self.apply_zone_change(cmd.split())
-            if(iterator_count > 0):
+            if (iterator_count > 0):
                 zone_with_sep += ';'
             iterator_count += 1
             zone_with_sep += zone
@@ -259,9 +259,9 @@ class BrcdFCZoneClientCLI(object):
             self._cfg_trans_abort()
             msg = _("Activating zone set failed: "
                     "(Zone set=%(cfg_name)s error=%(err)s)."
-                    ) % {'cfg_name': cfg_name, 'err': six.text_type(e)}
+                    ) % {'cfg_name': cfg_name, 'err': str(e)}
             LOG.error(msg)
-            raise exception.BrocadeZoningCliException(reason=msg)
+            raise b_exception.BrocadeZoningCliException(reason=msg)
 
     def activate_zoneset(self, cfgname):
         """Method to Activate the zone config. Param cfgname - ZonesetName."""
@@ -313,10 +313,10 @@ class BrcdFCZoneClientCLI(object):
                 self._cfg_save()
         except Exception as e:
             msg = _("Deleting zones failed: (command=%(cmd)s error=%(err)s)."
-                    ) % {'cmd': cmd, 'err': six.text_type(e)}
+                    ) % {'cmd': cmd, 'err': str(e)}
             LOG.error(msg)
             self._cfg_trans_abort()
-            raise exception.BrocadeZoningCliException(reason=msg)
+            raise b_exception.BrocadeZoningCliException(reason=msg)
 
     def get_nameserver_info(self):
         """Get name server data from fabric.
@@ -331,10 +331,10 @@ class BrcdFCZoneClientCLI(object):
                 'nsshow': zone_constant.NS_SHOW,
                 'nscamshow': zone_constant.NS_CAM_SHOW}
             cli_output = self._get_switch_info([cmd])
-        except exception.BrocadeZoningCliException:
+        except b_exception.BrocadeZoningCliException:
             with excutils.save_and_reraise_exception():
-                LOG.error(_LE("Failed collecting nsshow "
-                              "info for fabric %s"), self.switch_ip)
+                LOG.error("Failed collecting nsshow "
+                          "info for fabric %s", self.switch_ip)
         if (cli_output):
             return_list = self._parse_ns_output(cli_output)
         cli_output = None
@@ -349,7 +349,7 @@ class BrcdFCZoneClientCLI(object):
 
     def _cfg_trans_abort(self):
         is_abortable = self._is_trans_abortable()
-        if(is_abortable):
+        if (is_abortable):
             self.apply_zone_change([zone_constant.CFG_ZONE_TRANS_ABORT])
 
     def _is_trans_abortable(self):
@@ -360,12 +360,12 @@ class BrcdFCZoneClientCLI(object):
         output = stdout.splitlines()
         is_abortable = False
         for line in output:
-            if(zone_constant.TRANS_ABORTABLE in line):
+            if (zone_constant.TRANS_ABORTABLE in line):
                 is_abortable = True
                 break
         if stderr:
             msg = _("Error while checking transaction status: %s") % stderr
-            raise exception.BrocadeZoningCliException(reason=msg)
+            raise b_exception.BrocadeZoningCliException(reason=msg)
         else:
             return is_abortable
 
@@ -384,7 +384,7 @@ class BrcdFCZoneClientCLI(object):
                     "error=%(err)s).") % {'cmd': cmd_list, 'err': stdout}
             LOG.error(msg)
             self._cfg_trans_abort()
-            raise exception.BrocadeZoningCliException(reason=msg)
+            raise b_exception.BrocadeZoningCliException(reason=msg)
 
     def is_supported_firmware(self):
         """Check firmware version is v6.4 or higher.
@@ -405,13 +405,13 @@ class BrcdFCZoneClientCLI(object):
                             firmware = int(ver[0] + ver[1])
                 return firmware > 63
             else:
-                LOG.error(_LE("No CLI output for firmware version check"))
+                LOG.error("No CLI output for firmware version check")
                 return False
         except processutils.ProcessExecutionError as e:
             msg = _("Error while getting data via ssh: (command=%(cmd)s "
-                    "error=%(err)s).") % {'cmd': cmd, 'err': six.text_type(e)}
+                    "error=%(err)s).") % {'cmd': cmd, 'err': str(e)}
             LOG.error(msg)
-            raise exception.BrocadeZoningCliException(reason=msg)
+            raise b_exception.BrocadeZoningCliException(reason=msg)
 
     def _get_switch_info(self, cmd_list):
         stdout, stderr, sw_data = None, None, None
@@ -423,9 +423,9 @@ class BrcdFCZoneClientCLI(object):
         except processutils.ProcessExecutionError as e:
             msg = _("Error while getting data via ssh: (command=%(cmd)s "
                     "error=%(err)s).") % {'cmd': cmd_list,
-                                          'err': six.text_type(e)}
+                                          'err': str(e)}
             LOG.error(msg)
-            raise exception.BrocadeZoningCliException(reason=msg)
+            raise b_exception.BrocadeZoningCliException(reason=msg)
 
     def _parse_ns_output(self, switch_data):
         """Parses name server data.
@@ -436,7 +436,7 @@ class BrcdFCZoneClientCLI(object):
         """
         return_list = []
         for line in switch_data:
-            if not(" NL " in line or " N " in line):
+            if not (" NL " in line or " N " in line):
                 continue
             linesplit = line.split(';')
             if len(linesplit) > 2:
@@ -474,7 +474,7 @@ class BrcdFCZoneClientCLI(object):
                             command,
                             check_exit_code=check_exit_code)
                     except Exception as e:
-                        LOG.exception(_LE('Error executing SSH command.'))
+                        LOG.exception('Error executing SSH command.')
                         last_exception = e
                         greenthread.sleep(random.randint(20, 500) / 100.0)
                 try:
@@ -491,7 +491,7 @@ class BrcdFCZoneClientCLI(object):
                         cmd=command)
         except Exception:
             with excutils.save_and_reraise_exception():
-                LOG.error(_LE("Error running SSH command: %s"), command)
+                LOG.error("Error running SSH command: %s", command)
 
     def _ssh_execute(self, cmd_list, check_exit_code=True, attempts=1):
         """Execute cli with status update.
@@ -537,7 +537,7 @@ class BrcdFCZoneClientCLI(object):
                         else:
                             return True
                     except Exception as e:
-                        LOG.exception(_LE('Error executing SSH command.'))
+                        LOG.exception('Error executing SSH command.')
                         last_exception = e
                         greenthread.sleep(random.randint(20, 500) / 100.0)
                 LOG.debug("Handling error case after "
@@ -556,7 +556,7 @@ class BrcdFCZoneClientCLI(object):
                         cmd=command)
         except Exception as e:
             with excutils.save_and_reraise_exception():
-                LOG.error(_LE("Error executing command via ssh: %s"), e)
+                LOG.error("Error executing command via ssh: %s", e)
         finally:
             if stdin:
                 stdin.flush()
@@ -615,7 +615,7 @@ exit
             try:
                 channel.close()
             except Exception:
-                LOG.exception(_LE('Error closing channel.'))
+                LOG.exception('Error closing channel.')
             LOG.debug("_execute_cmd: stdout to return: %s", stdout)
             LOG.debug("_execute_cmd: stderr to return: %s", stderr)
         return (stdout, stderr)
